# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from mainstay import Mainstay, AsyncMainstay
from tests.utils import assert_matches_type
from mainstay.types.data import CrimeDataResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCrimeData:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_fetch(self, client: Mainstay) -> None:
        crime_data = client.data.crime_data._fetch(
            addresses=[
                {
                    "city": "San Antonio",
                    "postal_code": "78253",
                    "state": "TX",
                    "street": "14042 Kenyte Row",
                }
            ],
        )
        assert_matches_type(CrimeDataResponse, crime_data, path=["response"])

    @parametrize
    def test_raw_response_fetch(self, client: Mainstay) -> None:
        response = client.data.crime_data.with_raw_response._fetch(
            addresses=[
                {
                    "city": "San Antonio",
                    "postal_code": "78253",
                    "state": "TX",
                    "street": "14042 Kenyte Row",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        crime_data = response.parse()
        assert_matches_type(CrimeDataResponse, crime_data, path=["response"])

    @parametrize
    def test_streaming_response_fetch(self, client: Mainstay) -> None:
        with client.data.crime_data.with_streaming_response._fetch(
            addresses=[
                {
                    "city": "San Antonio",
                    "postal_code": "78253",
                    "state": "TX",
                    "street": "14042 Kenyte Row",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            crime_data = response.parse()
            assert_matches_type(CrimeDataResponse, crime_data, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCrimeData:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_fetch(self, async_client: AsyncMainstay) -> None:
        crime_data = await async_client.data.crime_data._fetch(
            addresses=[
                {
                    "city": "San Antonio",
                    "postal_code": "78253",
                    "state": "TX",
                    "street": "14042 Kenyte Row",
                }
            ],
        )
        assert_matches_type(CrimeDataResponse, crime_data, path=["response"])

    @parametrize
    async def test_raw_response_fetch(self, async_client: AsyncMainstay) -> None:
        response = await async_client.data.crime_data.with_raw_response._fetch(
            addresses=[
                {
                    "city": "San Antonio",
                    "postal_code": "78253",
                    "state": "TX",
                    "street": "14042 Kenyte Row",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        crime_data = await response.parse()
        assert_matches_type(CrimeDataResponse, crime_data, path=["response"])

    @parametrize
    async def test_streaming_response_fetch(self, async_client: AsyncMainstay) -> None:
        async with async_client.data.crime_data.with_streaming_response._fetch(
            addresses=[
                {
                    "city": "San Antonio",
                    "postal_code": "78253",
                    "state": "TX",
                    "street": "14042 Kenyte Row",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            crime_data = await response.parse()
            assert_matches_type(CrimeDataResponse, crime_data, path=["response"])

        assert cast(Any, response.is_closed) is True



# Standard Library
import json
from typing import List

# Third-Party Libraries
import pydantic.json

# 1st Party Libraries
import mainstay
from mainstay.types.data import rental_comps_response, rental_comps_fetch_params

# TODO: Rename rental_comps_fetch_params -> rental_comps_fetch_params

client = mainstay.Mainstay(
    # Set API key below or set the MAINSTAY_API_KEY environment variable.
    # api_key="my-api-key"
)

addresses: List[rental_comps_fetch_params.Address] = [
    {
        "street": "5201 South 44th St",
        "city": "Omaha",
        "state": "NE",
        "postal_code": "68107",
        "token": "client-provided-token-0",  # This client-provided token is optional and can be any string.
    }
]

filters: rental_comps_fetch_params.Filters = {
    "bedrooms_total": {
        "relative": 1,
    }
}

# Type hints
address: rental_comps_fetch_params.Address
result: rental_comps_response.Result

for (address, result) in zip(addresses, client.data.rental_comps.fetch(addresses=addresses, filters=filters)):
    print("input address:", address)
    print("token:", result.token)

    if result.subject_property_details:
        output_address = {
            "street": result.subject_property_details.street,
            "city": result.subject_property_details.city,
            "state": result.subject_property_details.state,
            "postal_code": result.subject_property_details.postal_code,
        }
        print("Output address:", output_address)
        print("Subject property details:", result.subject_property_details.model_dump_json(indent=2))
    print("Number of rental comps:", len(result.rental_comps))

    if result.rental_comps:
        print("Rental comps json:", json.dumps(result.rental_comps, indent=2, default=pydantic.json.pydantic_encoder))
        for index, rental_comp in enumerate(result.rental_comps):
            assert rental_comp.property_details
            print(f"Rental comp {index + 1}:")
            rental_comp_address = {
                "street": rental_comp.property_details.street,
                "city": rental_comp.property_details.city,
                "state": rental_comp.property_details.state,
                "postal_code": rental_comp.property_details.postal_code,
            }
            print("address:", rental_comp_address)
            print("close price:", rental_comp.close_price)
            print("close date:", rental_comp.close_price_date)
            print("distance miles:", rental_comp.distance_miles)
            print("dom:", rental_comp.dom)
            print("initial list price:", rental_comp.initial_list_price)
            print("initial list price date:", rental_comp.initial_list_price_date)
            print("last event date:", rental_comp.last_event_date)
            print("last list price:", rental_comp.last_list_price)
            print("listing status:", rental_comp.listing_status)
            print("move out date:", rental_comp.move_out_date)
            print("ownership profile:", rental_comp.ownership_profile)
            print("response codes:", rental_comp.response_codes)
            print("similarity score:", rental_comp.similarity_score, end="\n\n")
    else:
        print("No rental comps available for this address.")
